/*
 * scr_circle.c
 */

#include "screen.h"
#include <math.h>

/*
 * Draw a circle centered at (x0, y0) with radius r and width w in the given color.
 * Compute in the first quadrant only.
 * max is the largest x value to check for the current y value.
 */
void
scr_circle(x0, y0, r, w, color) int x0, y0, r, w, color;
{
	register int x, y, max, rem, irem, rsq, irsq;

	if (w >= r) {
		scr_fillcircle(x0, y0, r, color);
		return;
	}
	rsq = r * r;			/* radius squared */
	irsq = (r - w) * (r - w);	/* inner radius squared */
	for (y = 0, max = r; y <= r; y++) {
		rem = rsq - y * y;
		irem = (y > r - w) ? -1 : irsq - y * y;
		while (max * max > rem)
			--max;
		for (x = max; x * x > irem && x >= 0; --x) {
			scr_setcolor(x0 + x, y0 + y, color);
			scr_setcolor(x0 + x, y0 - y, color);
			scr_setcolor(x0 - x, y0 - y, color);
			scr_setcolor(x0 - x, y0 + y, color);
		}
	}
}

/* end of scr_circle.c */
