/*
 * scr_init.c
 */

#include <stdio.h>
#include "screen.h"
#include "vga.h"

#if	DEBUG
#define	srand(n)
#define	vga_setmode(mode)
#endif

/* Globals. */
char	*screen;			/* screen[src_ymax][scr_xmax]	*/
char	*scr_colors;			/* [0, src_ncolors] array of colors */
int	scr_mode;			/* graphics mode		*/
int	scr_ncolors;			/* number of colors		*/
int	scr_npixels;			/* number of pixels		*/
char	scr_version[] = SCR_VERSION;	/* version number		*/
int	scr_xmax;			/* x pixels on screen		*/
int	scr_ymax;			/* y pixels on screen		*/
int	scr_xymin;			/* min(scr_xmax, scr_ymax)	*/

int
scr_init(mode) int mode;
{
	register int i;

	switch(mode) {
	case G320x200x16:	scr_xmax = 320;	scr_ymax = 200;	scr_ncolors = 16;	break;
	case G640x200x16:	scr_xmax = 640;	scr_ymax = 200;	scr_ncolors = 16;	break;
	case G640x350x16:	scr_xmax = 640;	scr_ymax = 350;	scr_ncolors = 16;	break;
	case G640x480x16:	scr_xmax = 640;	scr_ymax = 480;	scr_ncolors = 16;	break;
	case G320x200x256:	scr_xmax = 320;	scr_ymax = 200;	scr_ncolors = 256;	break;
	case G320x240x256:	scr_xmax = 320;	scr_ymax = 240;	scr_ncolors = 256;	break;
	case G320x400x256:	scr_xmax = 320;	scr_ymax = 400;	scr_ncolors = 256;	break;
	case G360x480x256:	scr_xmax = 360;	scr_ymax = 480;	scr_ncolors = 256;	break;
	default:
		return 0;			/* unrecognized */
	}

	/*
	 * Clear the screen.
	 * This is done just to reduce the vertigo at exit.
	 * Cannot clear it to black on black because vga_init() catches <Ctrl-C>,
	 * hence scr_quit() cannot be guaranteed to restore it.
	 */
	printf("\033[2J");
	fflush(stdout);

	scr_mode = mode;
	vga_setmode(mode);
	scr_npixels = scr_xmax * scr_ymax;
	scr_xymin = (scr_xmax <= scr_ymax) ? scr_xmax : scr_ymax;
	if ((screen = scr_alloc()) == NULL)
		return 0;			/* screen allocation failed */
	if ((scr_colors = malloc(scr_ncolors)) == NULL)
		return 0;			/* colors array allocation failed */
	for (i = 0; i < scr_ncolors; i++)
		scr_colors[i] = i;		/* default color mapping */
	scr_clip(0, 0, scr_xmax - 1, scr_ymax - 1);
	tty_init();
	srand(time(NULL));
	return 1;
}

/* end of scr_init.c */
