/*
 * scr_line.c
 */

#include "screen.h"
#include <math.h>

/*
 * Draw a 1-pixel wide line of a given color.
 * FIX_ME This should be redone using Bresenham's algorithm.
 */
void
scr_line(ax, ay, bx, by, color) int ax, ay, bx, by, color;
{
	register int x, y, x0, y0, xdist, ydist, abs_ydist;
	double len;

	/* Degenerate line: one pixel. */
	if (ax == bx && ay == by) {
		scr_setcolor(ax, ay, color);
		return;
	}

	/* Flip a and b if necessary to force ax <= bx. */
	if (ax > bx) {
		x = ax;
		ax = bx;
		bx = x;
		y = ay;
		ay = by;
		by = y;
	}

	/* Compute x and y differences; at least one of xdist, ydist is nonzero. */
	xdist = bx - ax;			/* N.B. xdist >= 0 */
	abs_ydist = ydist = by - ay;
	if (ydist < 0)
		abs_ydist = -ydist;

	/* Compute the line length.  Avoid the sqrt() if horizontal or verical. */
	if (xdist == 0)
		len = abs_ydist;
	else if (ydist == 0)
		len = xdist;
	else
		len = sqrt((double)(xdist * xdist + ydist * ydist));

	/* Render the line. */
	if (xdist >= abs_ydist) {
		/*
		 * If xdist >= abs_ydist, render the line as a horizontal sequence.
		 * Note that xdist != 0 in this case because at least one
		 * of xdist and ydist is nonzero.
		 */
		for (x = 0; x <= xdist; x++) {
			x0 = ax + x;
			y0 = ay + x * ydist / xdist;
			scr_setcolor(x0, y0, color);
		}
	} else {				/* abs_ydist < xdist */
		/*
		 * If xdist < abs_ydist, render the line as a vertical sequence.
		 * Note that ydist != 0 in this case because at least one
		 * of xdist and ydist is nonzero.
		 */
		for (y = 0; y <= abs_ydist; y++) {
			x0 = ax + y * xdist / abs_ydist;
			y0 = (ydist >= 0) ? ay + y : ay - y;
			scr_setcolor(x0, y0, color);
		}
	}
}

/* end of scr_line.c */
