/*
 * scr_wline.c
 */

#include "screen.h"
#include <math.h>

/*
 * Draw a line of a given width and color.
 * The rendered line is w pixels wide, centered on the end points.
 * This is code adjusts the width w of the rendered line
 * depending on the angle of the line.
 * Horizontal and vertical lines of the same width have different
 * apparent widths if pixel width != pixel height.
 * FIX_ME This should be redone using Bresenham's algorithm.
 */
void
scr_wline(ax, ay, bx, by, width, color) int ax, ay, bx, by, width, color;
{
	register int x, y, x0, y0, xdist, ydist, abs_ydist, w, w2;
	double len;

	if (width <= 1) {
		if (width == 1)
			scr_line(ax, ay, bx, by, color);
		return;
	}

	/* Degenerate line: one pixel by desired width. */
	if (ax == bx && ay == by) {
		x0 = ax - width / 2;
		for (x = 0; x < width; x++)
			scr_setcolor(x0 + x, ay, color);
		return;
	}

	/* Flip a and b if necessary to force ax <= bx. */
	if (ax > bx) {
		x = ax;
		ax = bx;
		bx = x;
		y = ay;
		ay = by;
		by = y;
	}

	/* Compute x and y differences; at least one of xdist, ydist is nonzero. */
	xdist = bx - ax;			/* N.B. xdist >= 0 */
	abs_ydist = ydist = by - ay;
	if (ydist < 0)
		abs_ydist = -ydist;

	/* Compute the line length.  Avoid the sqrt() if horizontal or verical. */
	if (xdist == 0)
		len = abs_ydist;
	else if (ydist == 0)
		len = xdist;
	else
		len = sqrt((double)(xdist * xdist + ydist * ydist));

	/* Render the line. */
	if (xdist >= abs_ydist) {
		/*
		 * If xdist >= abs_ydist, render the line as a horizontal sequence
		 * of vertical pixel segments.
		 * Note that xdist != 0 in this case because at least one
		 * of xdist and ydist is nonzero.
		 */
		w = (int)(.5 + width * len / xdist);
		w2 = w / 2;
		for (x = 0; x <= xdist; x++) {
			x0 = ax + x;
			y0 = ay - w2 + x * ydist / xdist;
			for (y = 0; y < w; y++ ) {
				scr_setcolor(x0, y0 + y, color);
			}
		}
	} else {				/* abs_ydist < xdist */
		/*
		 * If xdist < abs_ydist, render the line as a vertical sequence
		 * of horizontal pixel segments.
		 * Note that ydist != 0 in this case because at least one
		 * of xdist and ydist is nonzero.
		 */
		w = (int)(.5 + width * len / abs_ydist);
		w2 = w / 2;
		for (y = 0; y <= abs_ydist; y++) {
			x0 = ax - w2 + y * xdist / abs_ydist;
			y0 = (ydist >= 0) ? ay + y : ay - y;
			for (x = 0; x < w; x++) {
				scr_setcolor(x0 + x, y0, color);
			}
		}
	}
}

/* end of scr_wline.c */
