/*
 * screen.h
 * Screen library header for use with libvga.a.
 * By Stephen A. Ness (steve@mwc.com).
 */

#include <stdlib.h>

#define	SCR_VERSION	"libscreen: V1.4"

#if	__STDC__
#define	__(args)	args
#else
#define	__(args)	()
#endif

/* Library globals. */
extern	char	*screen;		/* screen[src_ymax][scr_xmax]	*/
extern	char	*screen0;		/* &screen[src_y0][scr_x0]	*/
extern	char	*scr_colors;		/* [0, src_ncolors] array of colors */
extern	int	scr_lrx;		/* lrx of clipping window	*/
extern	int	scr_lry;		/* lry of clipping window	*/
extern	int	scr_mode;		/* graphics mode		*/
extern	int	scr_ncolors;		/* number of colors		*/
extern	int	scr_npixels;		/* number of pixels		*/
extern	int	scr_ulx;		/* ulx of clipping window	*/
extern	int	scr_uly;		/* uly of clipping window	*/
extern	char	scr_version[];		/* version number		*/
extern	int	scr_x0;			/* x of center of screen	*/
extern	int	scr_xdim;		/* x pixels on window		*/
extern	int	scr_xmax;		/* x pixels on screen		*/
extern	int	scr_xymax;		/* max(scr_xdim, scr_ydim)	*/
extern	int	scr_xymin;		/* min(scr_xmax, scr_ymax)	*/
extern	int	scr_y0;			/* y of center of screen	*/
extern	int	scr_ydim;		/* y pixels on window		*/
extern	int	scr_ymax;		/* y pixels on screen		*/

/* Macros. */
#define	scr_addr(x, y)		(screen + (y) * scr_xmax + (x))
#define	scr0_addr(x, y)		(screen0 + (y) * scr_xmax + (x))
#define	scr_alloc()		malloc(scr_npixels)
#define	scr_clear(color)	memset(screen, color, scr_npixels)
#define	scr_color(x, y)		*scr_addr((x), (y))
#define	scr0_color(x, y)	*scr0_addr((x), (y))
#define	scr_inwindow(x, y)	((x) >= scr_ulx && (x) <= scr_lrx && (y) >= scr_uly && (y) <= scr_lry)
#define	scr_pline(x, y, r, theta, w, c)	scr_wline((x), (y), scr_ptox((x), (r), (theta)), scr_ptoy((y), (r), (theta)), (w), (c))
#define	scr_pline0(r, theta, w, c)	scr_pline(scr_x0, scr_y0, (r), (theta), (w), (c))
#define	scr_ptox(x, len, theta)	((x) + (int)(.5 + (len) * cos(theta)))
#define	scr_ptoy(y, len, theta)	((y) + (int)(.5 + (len) * sin(theta)))
#if	DEBUG
#define	scr_quit()
#else
#define	scr_quit()	vga_setmode(TEXT)
#endif
#define	scr_setcolor(x, y, c)	if (scr_inwindow((x), (y))) scr_color((x), (y)) = (c)

/* Types. */
/* Optical effect types for scr_effect(). */
typedef	enum	effect
{
	RANDOM,					/* one of the following	*/
	DOWN,	UP,	UDTOC,	CTOUD,		/* horizontal wipes	*/
	LTOR,	RTOL,	LRTOC,	CTOLR,		/* vertical wipes	*/
	UDN,	LRN,	UDSQN,	LRSQN,		/* n-window wipes	*/
	DIAGUL,	DIAGUR,	DIAGLL,	DIAGLR,		/* diagonal wipes	*/
	SQIN,	SQOUT,				/* squares		*/
	DIAGIN,	DIAGOUT, DSQN,			/* diagonal squares	*/
	WW1,	WW2,	WW3,			/* windshield washers	*/
	SPIN,	SPIN2,				/* spirals		*/
	CIN,	COUT,	CSQIN,	CSQOUT,		/* circles		*/
	CLR,	CUD,	CSQ,	CSQSQ,		/* compound effects	*/
	DISS1,	DISS2,	DISS3,	DISSSQ,		/* dissolves		*/
	FADEIN, FADEOUT				/* fades		*/
} EFFECT;
#define	NWIPES		SPIN2			/* number of wipes	*/
#define	NEFFECTS	FADEOUT			/* number of effects	*/

/* Functions. */
extern	void	scr_circle	__((int x0, int y0, int r, int width, int color));
extern	void	scr_clip	__((int ulx, int uly, int lrx, int lry));
extern	void	scr_display	__((int flag));
extern	int	scr_effect	__((EFFECT effect));
extern	void	scr_fillcircle	__((int x0, int y0, int r, int color));
extern	void	scr_fillrectangle __((int ulx, int uly, int lrx, int lry, int color));
extern	int	scr_init	__((int mode));
extern	void	scr_line	__((int ax, int ay, int bx, int by, int color));
extern	void	scr_pixel	__((int x, int y));
extern	void	scr_recolor	__((void));
extern	void	scr_rectangle	__((int ulx, int uly, int lrx, int lry, int width, int color));
extern	void	scr_wline	__((int ax, int ay, int bx, int by, int width, int color));
#if	DEBUG
#define	tty_check()	1
#define	tty_init()
#else
extern	int	tty_check	__((void));
extern	void	tty_init	__((void));
#endif

/* end of screen.h */
